package org.seasar.struts.extension.customizer;

import java.util.List;

import org.apache.commons.validator.Arg;
import org.apache.commons.validator.Field;
import org.apache.commons.validator.Form;
import org.apache.commons.validator.Var;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.container.ComponentDef;
import org.seasar.struts.config.S2ActionMapping;
import org.seasar.struts.customizer.ActionCustomizer;
import org.seasar.struts.extension.config.XActionMapping;
import org.seasar.struts.util.ActionUtil;
import org.seasar.struts.validator.S2ValidatorResources;

/**
 * {@link ActionCustomizer} の拡張クラスです。
 *
 * @author awaawa
 *
 */
public class XActionCustomizer extends ActionCustomizer {

	/** インデックスの引数位置。 */
	private static final int INDEX_ARG_POSITION = 5;

	/**
	 * {@link XActionMapping} を作成します。
	 * @param componentDef componentDef
	 */
	@Override
	protected S2ActionMapping createActionMapping(ComponentDef componentDef) {
		XActionMapping actionMapping = new XActionMapping();
		actionMapping.setPath(ActionUtil.fromActionNameToPath(componentDef
				.getComponentName()));
		actionMapping.setComponentDef(componentDef);
		actionMapping.setName(componentDef.getComponentName() + "Form");
		Class<?> actionClass = componentDef.getComponentClass();
		setupActionForm(actionMapping, actionClass);
		setupMethod(actionMapping, actionClass);
		return actionMapping;
	}

	/**
	 * {@inheritDoc}
	 * ActionFormのプロパティが配列やリストの場合でも。正しく validate の設定をします。
	 */
	@Override
	protected void setupValidator(
			S2ActionMapping actionMapping,
			S2ValidatorResources validatorResources) {

		super.setupValidator(actionMapping, validatorResources);

		// TODO パフォーマンス的に 以下の2つの for 文のどちらを先にしたほうがいいか考えること。
		// とりあえず、現状は配列 or java.util.List のみ。
		BeanDesc beanDesc = actionMapping.getActionFormBeanDesc();
		for (String methodName : actionMapping.getExecuteMethodNames()) {
			if (!actionMapping.getExecuteConfig(methodName).isValidator()) {
				continue;
			}

			String formKey = actionMapping.getName() + "_" + methodName;
			Form form = validatorResources.getForm(null, formKey);

			@SuppressWarnings("unchecked")
			List<Field> list = form.getFields();

			for (Field field : list) {
				String property = field.getProperty();

				if (property == null) {
					property = field.getIndexedListProperty();
				}

				if (property == null) {
					continue;
				}

				Class<?> type
						= beanDesc.getPropertyDesc(property).getPropertyType();

				for (String suffix : new String[]{"s", "List"}) {
					String combiProperty;
					if ((type.isArray() || type.equals(List.class))
							&& property.endsWith(suffix)) {
						int endIndex = property.length() - suffix.length();
						combiProperty = property.substring(0, endIndex);
					} else {
						combiProperty = property + suffix;
					}

					if (combiProperty != null
							&& form.getField(combiProperty) != null) {
						field.addVar("startIndex", "1", Var.JSTYPE_INT);
						break;
					}
				}

				if (type.isArray() || type.equals(List.class)) {

					if (!"required".equals(field.getDepends())) {
						field.setIndexedListProperty(property);
						field.setProperty(null);
						field.generateKey();

						String trimStr = ".null";
						String key = field.getKey();
						if (key != null && key.endsWith(trimStr)) {
							int endIndex = key.length() - trimStr.length();
							field.setKey(key.substring(0, endIndex));
						}
					}
				}

				Arg[] args = field.getArgs(field.getDepends());
				int position = INDEX_ARG_POSITION;
				if (args != null && args.length > position) {
					position = args.length;
				}

				Arg arg = new Arg();
				arg.setKey("errors.index");
				arg.setResource(true);
				arg.setPosition(position);
				field.addArg(arg);
			}
		}
	}
}
